﻿local L = LibStub("AceLocale-3.0"):GetLocale("GigaItemAuction", false)

local _G				= getfenv(0)

local pairs			    = _G.pairs
local ipairs			= _G.ipairs
local select			= _G.select

local table_insert		= _G.table.insert
local table_remove		= _G.table.remove
local table_sort		= _G.table.sort

local GuildControlGetRankName 	= _G.GuildControlGetRankName
local GetGuildRosterInfo		= _G.GetGuildRosterInfo
local GetNumGuildMembers		= _G.GetNumGuildMembers
local GetItemInfo				= _G.GetItemInfo

local anchorAdmin 		= "GIAAdminFrame"

local PLAYERNAME 		= UnitName("player")
local _, PLAYERCLASS 	= UnitClass("player")
local PLAYERGUID 		= 0

local db

local WITHDRAWAPPROVE	= 1
local WITHDRAWBLOCK		= 2
local ANONYMHIDE		= 1
local ANONYMVISIBLE		= 2

local function getGameTimeAsUnix()
	return time()
end

local function getCurrentDate(timestamp, short)
   local dt = date("*t", timestamp)
   if short then
      return string.format("%02d:%02d:%02d", dt.hour, dt.min, dt.sec)
   else
      return string.format("%02d/%02d/%d - %02d:%02d:%02d", dt.day , dt.month, dt.year, dt.hour, dt.min, dt.sec)
   end
end

function GigaItemAuction:ADMINOnInitialize()
	db 			= self.db.profile
end

function GigaItemAuction:CreateADMINFrame()
	PLAYERGUID 	= self:getMyGUID()

	if self.ADMINFrame then return end
	
	local dbn = db.nowAuction
	local dbc = db.config
	local dba = db.admin
	
	local sfont = "Interface\\AddOns\\GigaItemAuction\\textures\\FRIZQT__.TTF"
	
	self.ADMINFrame = CreateFrame("Frame", anchorAdmin, UIParent)
    local f = self.ADMINFrame
	local m = self.AUCTIONFrame
	
    f:SetResizable(false)
    f:SetMovable(true)
    f:SetWidth(m:GetWidth())
    f:SetHeight(m:GetHeight())
    f:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background",
        edgeFile =  "Interface/Tooltips/UI-Tooltip-Border",
        edgeSize= 16, insets = { left = 0, right = 0, top = 0, bottom = 0 }}) 
    f:SetBackdropColor(unpack(self.colors.black_trans))
    f:SetBackdropBorderColor(unpack(self.colors.red_trans))
    f:EnableMouse(true)
    f:ClearAllPoints()
    f:SetScript("OnMouseDown", function() f:StartMoving(); end)
    f:SetScript("OnMouseUp", function() f:StopMovingOrSizing(); end)
    f:SetPoint("LEFT", m, "RIGHT")
	
--~ 	Titel
	f.Title = f:CreateFontString("GIA_TitleAdmin", "HIGH")
    f.Title:SetFont(sfont,14)
    f.Title:SetText(L["AuctionControlPanel"])
    f.Title:SetPoint("TOPLEFT", f, "TOPLEFT", 10, -5)
    f.Title:SetTextColor(unpack(self.colors.yellow))
	
--~     closeButton
    f.closeButton = CreateFrame("Button", nil, f, "UIPanelCloseButton")
    f.closeButton:SetPoint("TOPRIGHT", f, "TOPRIGHT", 3, 3)
    f.closeButton:SetWidth(30)
    f.closeButton:SetHeight(30)    
    f.closeButton:SetScript("OnClick", function() self:HideADMINFrame() end)
	
--~     adminButton
    f.adminButton = CreateFrame("Button", nil, f, "UIPanelButtonTemplate")
    f.adminButton:SetPoint("RIGHT", f.closeButton, "LEFT", 5, 0)
    f.adminButton:SetWidth(16)
    f.adminButton:SetHeight(16)  
    f.adminButton:SetText("L") 
	f.adminButton:RegisterForClicks("AnyUp")
    f.adminButton:SetScript("OnClick", function() self:ToggleBIDLISTFrame() end)
	
	
	
-------------------------------------------------------------------
--~ 	AktuellerAuctionOptionen
	f.breakTitle = f:CreateFontString("GIA_breakTitle", "HIGH")
    f.breakTitle:SetFont(sfont,14)
    f.breakTitle:SetText(L["Current auction:"])
    f.breakTitle:SetPoint("TOPLEFT", f, "TOPLEFT", 10, -25)
    f.breakTitle:SetTextColor(unpack(self.colors.blue_light))
	
--~     pauseButton
    f.pauseButton = CreateFrame("Button", nil, f, "UIPanelButtonTemplate")
    f.pauseButton:SetPoint("TOPLEFT", f.breakTitle, "BOTTOMLEFT", -2, -5)
    f.pauseButton:SetWidth(43)
    f.pauseButton:SetHeight(20)  
    f.pauseButton:SetText(L["admin_button_text_pause"]) 
	f.pauseButton:RegisterForClicks("AnyUp")
    f.pauseButton:SetScript("OnClick", function() self:SendAdminPauseAuction() end)
	f.pauseButton:SetScript("OnEnter", function(self)
		GameTooltip:SetOwner(f, "ANCHOR_TOPLEFT",0,5);
		GameTooltip:AddLine(L["tooltip_admin_pause"])
		GameTooltip:Show()
	end)
	f.pauseButton:SetScript("OnLeave", function(self) GameTooltip:Hide() end)
	f.pauseButton:Disable()
	
--~     startButton
    f.startButton = CreateFrame("Button", nil, f, "UIPanelButtonTemplate")
    f.startButton:SetPoint("LEFT", f.pauseButton, "RIGHT", 2, 0)
    f.startButton:SetWidth(43)
    f.startButton:SetHeight(20)  
    f.startButton:SetText(L["admin_button_text_start"]) 
	f.startButton:RegisterForClicks("AnyUp")
    f.startButton:SetScript("OnClick", function() self:SendAdminStartAuction() end)
	f.startButton:SetScript("OnEnter", function(self)
		GameTooltip:SetOwner(f, "ANCHOR_TOPLEFT",0,5);
		GameTooltip:AddLine(L["tooltip_admin_start"])
		GameTooltip:Show()
	end)
	f.startButton:SetScript("OnLeave", function(self) GameTooltip:Hide() end)
	f.startButton:Disable()
	
--~     resetButton
    f.resetButton = CreateFrame("Button", nil, f, "UIPanelButtonTemplate")
    f.resetButton:SetPoint("LEFT", f.startButton, "RIGHT", 2, 0)
    f.resetButton:SetWidth(43)
    f.resetButton:SetHeight(20)  
    f.resetButton:SetText(L["admin_button_text_reset"]) 
	f.resetButton:RegisterForClicks("AnyUp")
    f.resetButton:SetScript("OnClick", function() self:SendAdminResetAuction() end)
	f.resetButton:SetScript("OnEnter", function(self)
		GameTooltip:SetOwner(f, "ANCHOR_TOPLEFT",0,5);
		GameTooltip:AddLine(L["tooltip_admin_reset"])
		GameTooltip:Show()
	end)
	f.resetButton:SetScript("OnLeave", function(self) GameTooltip:Hide() end)
	f.resetButton:Disable()
	
--~     breakButton
    f.breakButton = CreateFrame("Button", nil, f, "UIPanelButtonTemplate")
    f.breakButton:SetPoint("LEFT", f.resetButton, "RIGHT", 2, 0)
    f.breakButton:SetWidth(67)
    f.breakButton:SetHeight(20)  
    f.breakButton:SetText(L["admin_button_text_break"]) 
	f.breakButton:RegisterForClicks("AnyUp")
    f.breakButton:SetScript("OnClick", function() self:SendAdminBreak() end)
	f.breakButton:SetScript("OnEnter", function(self)
		GameTooltip:SetOwner(f, "ANCHOR_TOPLEFT",0,5);
		GameTooltip:AddLine(L["tooltip_admin_break"])
		GameTooltip:Show()
	end)
	f.breakButton:SetScript("OnLeave", function(self) GameTooltip:Hide() end)
	f.breakButton:Disable()
	
	
	
--------------------------------------------------
--~ 	SliderText
	f.SliderText = f:CreateFontString("GIA_breakTitle", "HIGH")
    f.SliderText:SetFont(sfont,14)
    f.SliderText:SetText(L["Bidtime:"])
    f.SliderText:SetPoint("TOPLEFT", f, "TOPLEFT", 10, -70)
    f.SliderText:SetTextColor(unpack(self.colors.blue_light))
	
--~ 	Slider
	f.timeSlider = CreateFrame("Slider", "GIA_timeSlider", f, "OptionsSliderTemplate")
    f.timeSlider:SetPoint("TOPLEFT", f.SliderText, "BOTTOMLEFT", 0, -9)
	f.timeSlider:SetWidth(127)
	f.timeSlider:SetHeight(20)
	f.timeSlider:SetOrientation('HORIZONTAL')
	f.timeSlider:SetMinMaxValues(10,120)
	f.timeSlider:SetValue(dba.bidTime)
	f.timeSlider:SetValueStep(5)
	f.timeSlider:SetScript("OnValueChanged", function()
		getglobal(f.timeSlider:GetName() .. 'Text'):SetText(this:GetValue()..L["sec"])
	end)
	f.timeSlider:SetScript("OnMouseUp", function() self:SendBidTimeToAll(this:GetValue()) end)
	
	f.timeSlider.tooltipText = L["Bidtime (sec.)"]
	getglobal(f.timeSlider:GetName() .. 'Low'):SetText('10');
	getglobal(f.timeSlider:GetName() .. 'High'):SetText('120'); 
	getglobal(f.timeSlider:GetName() .. 'Text'):SetText(dba.bidTime..L["sec"]);
	
--~     aLongButton
    f.aLongButton = CreateFrame("Button", nil, f, "UIPanelButtonTemplate")
    f.aLongButton:SetPoint("LEFT", f.timeSlider, "RIGHT", 5, 0)
    f.aLongButton:SetWidth(67)
    f.aLongButton:SetHeight(20)  
    f.aLongButton:SetText(L["admin_button_text_timeedit"]) 
	f.aLongButton:RegisterForClicks("AnyUp")
    f.aLongButton:SetScript("OnClick", function() self:SendAdminAuctionLonger() end)
	f.aLongButton:SetScript("OnEnter", function(self)
		GameTooltip:SetOwner(f, "ANCHOR_TOPLEFT",0,5);
		GameTooltip:AddLine(L["tooltip_admin_timeedit"])
		GameTooltip:Show()
	end)
	f.aLongButton:SetScript("OnLeave", function(self) GameTooltip:Hide() end)
	f.aLongButton:Disable()
	
	
	
--------------------------------------------------
--~ 	BietKnöpfeKontrolleText
	f.BidWithdrawText = f:CreateFontString("GIA_BidWithdrawTitle", "HIGH")
    f.BidWithdrawText:SetFont(sfont,14)
    f.BidWithdrawText:SetText(L["Withdraw bid"]..":")
    f.BidWithdrawText:SetPoint("TOPLEFT", f, "TOPLEFT", 10, -120)
    f.BidWithdrawText:SetTextColor(unpack(self.colors.blue_light))
	
	
--~     RadioWithdrawButton  
    if not f.RadioWithdraw then f.RadioWithdraw = {} end
    if not f.RadioWithdrawText then f.RadioWithdrawText = {} end
    for i = 1, 2 do
        f.RadioWithdraw[i] = CreateFrame("CheckButton", "GIAAdminRadioWithdraw"..i.."Setup", f, "UICheckButtonTemplate")
		f.RadioWithdraw[i]:SetWidth(16)
		f.RadioWithdraw[i]:SetHeight(16)
        f.RadioWithdrawText[i] = f:CreateFontString("GIAAdminRadioWithdraw"..i.."FrameTitle", "HIGH")
		
        if i == 1 then
            f.RadioWithdraw[i]:SetPoint("TOPLEFT", f.BidWithdrawText, "BOTTOMLEFT", 0, -5)
        else
            f.RadioWithdraw[i]:SetPoint("LEFT", f.RadioWithdrawText[i-1], "RIGHT", 5, 0)
        end
		
		if dba.bidWithdraw and i == WITHDRAWAPPROVE then
            f.RadioWithdraw[i]:SetChecked(true)
		elseif not dba.bidWithdraw and i == WITHDRAWBLOCK then
            f.RadioWithdraw[i]:SetChecked(true)
		end
        
        f.RadioWithdraw[i]:SetScript("OnClick", function() self:ToggleAdminRadioWithdraw(i) end)
        f.RadioWithdraw[i]:SetText(i)
		
        f.RadioWithdrawText[i]:SetPoint("LEFT", f.RadioWithdraw[i], "RIGHT", 0, 0)
        f.RadioWithdrawText[i]:SetFont(sfont,14, "OUTLINE")
        f.RadioWithdrawText[i]:SetText(i == WITHDRAWAPPROVE and L["approve"] or L["block"])
        f.RadioWithdrawText[i]:SetTextColor(unpack(self.colors.white))
    end
	
	
	
--------------------------------------------------
--~ 	HöchstbieterKontrolleText
	f.HighestBidderText = f:CreateFontString("GIA_HighestBidderTitle", "HIGH")
    f.HighestBidderText:SetFont(sfont,14)
    f.HighestBidderText:SetText(L["Highest bidder:"])
    f.HighestBidderText:SetPoint("TOPLEFT", f, "TOPLEFT", 10, -170)
    f.HighestBidderText:SetTextColor(unpack(self.colors.blue_light))
	
	
--~     RadioWithdrawButton  
    if not f.RadioHighestBidder then f.RadioHighestBidder = {} end
    if not f.RadioHighestBidderText then f.RadioHighestBidderText = {} end
    for i = 1, 2 do
        f.RadioHighestBidder[i] = CreateFrame("CheckButton", "GIAAdminRadioHighestBidder"..i.."Setup", f, "UICheckButtonTemplate")
		f.RadioHighestBidder[i]:SetWidth(16)
		f.RadioHighestBidder[i]:SetHeight(16)
        f.RadioHighestBidderText[i] = f:CreateFontString("GIAAdminRadioHighestBidder"..i.."FrameTitle", "HIGH")
		
        if i == 1 then
            f.RadioHighestBidder[i]:SetPoint("TOPLEFT", f.HighestBidderText, "BOTTOMLEFT", 0, -5)
        else
            f.RadioHighestBidder[i]:SetPoint("LEFT", f.RadioHighestBidderText[i-1], "RIGHT", 5, 0)
        end
		
		if not dba.bidderShow and i == ANONYMHIDE then
            f.RadioHighestBidder[i]:SetChecked(true)
		elseif dba.bidderShow and i == ANONYMVISIBLE then
            f.RadioHighestBidder[i]:SetChecked(true)
		end
        
        f.RadioHighestBidder[i]:SetScript("OnClick", function() self:ToggleAdminRadioHighestBidder(i) end)
        f.RadioHighestBidder[i]:SetText(i)
		
        f.RadioHighestBidderText[i]:SetPoint("LEFT", f.RadioHighestBidder[i], "RIGHT", 0, 0)
        f.RadioHighestBidderText[i]:SetFont(sfont,14, "OUTLINE")
        f.RadioHighestBidderText[i]:SetText(i == ANONYMHIDE and L["anonym"] or ("|cffff5555"..L["visible for all"].."|r"))
        f.RadioHighestBidderText[i]:SetTextColor(unpack(self.colors.white))
    end
	
--------------------------------------------------
	self:CreateBIDLISTFrame()
	
	f:Hide()
end

function GigaItemAuction:CreateBIDLISTFrame()
    
	if self.BIDLISTFrame then return end
	
	local sfont = "Interface\\AddOns\\GigaItemAuction\\textures\\FRIZQT__.TTF"
    local a = self.ADMINFrame
	local m = self.AUCTIONFrame
	
	self.BIDLISTFrame = CreateFrame("Frame", anchorAdmin.."BIDLIST", a)
    local f = self.BIDLISTFrame
	
	f.isResizing = false
	
	f:SetResizable(true)
    f:SetMovable(true)
    f:SetWidth(a:GetWidth())
    f:SetHeight(a:GetHeight())
	f:SetMinResize(a:GetWidth(),a:GetHeight())
    f:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background",
        edgeFile =  "Interface/Tooltips/UI-Tooltip-Border",
        edgeSize= 16, insets = { left = 0, right = 0, top = 0, bottom = 0 }}) 
    f:SetBackdropColor(unpack(self.colors.black_trans))
    f:SetBackdropBorderColor(unpack(self.colors.red_trans))
    f:ClearAllPoints()
    f:SetPoint("LEFT", a, "RIGHT")
	
--~ 	Titel
	f.Title = f:CreateFontString("GIA_Title3", "HIGH")
    f.Title:SetFont(sfont,14)
    f.Title:SetText(L["Bidlogbook"])
    f.Title:SetPoint("TOPLEFT", f, "TOPLEFT", 10, -5)
    f.Title:SetTextColor(unpack(self.colors.yellow))
	
	
    f.scroll = CreateFrame("ScrollFrame", "GIAScrollFrame", f, "UIPanelScrollFrameTemplate")
	f.scroll:SetPoint("TOPLEFT", f, "TOPLEFT", 10, -25)
	f.scroll:SetPoint("RIGHT", f, "RIGHT", -25, 0)
	f.scroll:SetPoint("BOTTOM", f, 0, 10)

	f.EditBox = CreateFrame("EditBox", "GIAEditBox", f.scroll)
	f.EditBox:SetFont(sfont,14)
	f.EditBox:SetMultiLine(true)
	f.EditBox:SetWidth(a:GetWidth()-40)
	f.EditBox:SetScript("OnEscapePressed", function() this:ClearFocus() end)
	f.EditBox:SetScript("OnTextChanged", function() f.scroll:UpdateScrollChildRect() end)
    f.EditBox:SetScript("OnEditFocusGained", function() this:ClearFocus() end)
	f.EditBox:SetText("")
	f.EditBox:SetAutoFocus(false)
	f.EditBox:ClearFocus()
	f.scroll:SetScrollChild(f.EditBox)
	
	
	f.DragRight = CreateFrame("Button", nil, f)
	f.DragRight:SetFrameLevel( f:GetFrameLevel() + 10)
	f.DragRight:SetNormalTexture("Interface\\AddOns\\GigaItemAuction\\textures\\ResizeGripRight")
	f.DragRight:SetHighlightTexture("Interface\\AddOns\\GigaItemAuction\\textures\\ResizeGripRight")
	f.DragRight:SetWidth(16)
	f.DragRight:SetHeight(16)
	f.DragRight:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", 5, -5)
	f.DragRight:EnableMouse(true)
	f.DragRight:SetScript("OnMouseDown", 
        function() 
            f.isResizing = true; 
			f:StartSizing("BOTTOMRIGHT") 
        end )
	f.DragRight:SetScript("OnMouseUp", 
        function() 
            if f.isResizing == true then 
                f:StopMovingOrSizing();
				f:SetPoint("LEFT", a, "RIGHT")
				f:SetWidth(f:GetWidth())
				f:SetHeight(f:GetHeight())
                f.isResizing = false;
            end 
        end )

end


function GigaItemAuction:UpdateBidList()

	if not self:CheckSecurity(PLAYERNAME, 'auctioneer') then return end
	
	local dbn = db.nowAuction
	local dbc = db.config
	local dba = db.admin
	
    local f = self.BIDLISTFrame
	local text, player, class = "", "", ""
	local cache = dbn.cache
	
	if PLAYERNAME ~= dbn.auctionAuthor then
		player, class = self:GetGuildPlayer(dbn.auctionAuthor)
		if player ~= L["unknown"] then
			text = L["onlySee"]:format("|cff"..self:GetClassHexColor(class)..player.."|r")
		else
			text = L["onlySee"]:format(dbn.auctionAuthor or L["unknown"])
		end
	elseif #cache > 0 then
		table_sort(cache,	function(w1,w2)
								if w1["BidTime"] > w2["BidTime"] then
									return true
								end
							end)
		
		for i, p in pairs(cache) do
			player, class = self:GetGuildPlayer(p.BidNameID)
			text = text .. "|cff555555["..getCurrentDate(p.BidTime, true).."]|r "
			text = text .. "|cff"..self:GetClassHexColor(class)
			text = text .. p.BidNameID.."|r "
			text = text .. p.BidPoint
			if p.BidMethod == "G" then
				text = text .. " |cff55ff55"..(p.BidArt or "X")
				text = text .. (p.BidU80 and ("|r |cffff5555("..self.BidArt.kurz[4]..")|r") or "").."|r\n"
			else
				text = text .. " |cffff5555PASS|r\n"
			end
		end
		
	end
	
	f.EditBox:SetText(text)
end
    
    
----------------------------------------------------------
--~ Show/Hide/Save Window

function GigaItemAuction:ToggleAdminRadioWithdraw(radio)
	local f = self.ADMINFrame
	local dba = db.admin
	
    for j = 1, 2 do
        f.RadioWithdraw[j]:SetChecked(false)
    end
    
    for i = 1, 2 do
        if radio == i then
            f.RadioWithdraw[i]:SetChecked(true)
            dba.bidWithdraw = (i == WITHDRAWAPPROVE) and true or false
        end
    end
	
	if not dba.bidWithdraw then
		self:SetBreakButtonDisable()
	end
	
--~ 	self:SendAdminBlockWithdraw()
	self:SendAdminEditOption('bidWithdraw')
end

function GigaItemAuction:ToggleAdminRadioHighestBidder(radio)
	local f = self.ADMINFrame
	local dba = db.admin
	local dbc = db.config
	
    for j = 1, 2 do
        f.RadioHighestBidder[j]:SetChecked(false)
    end	
    
    for i = 1, 2 do
        if radio == i then
            f.RadioHighestBidder[i]:SetChecked(true)
        end
    end
	
	dba.bidderShow = (radio == ANONYMVISIBLE) and true or false
	
	self:Debug("ToggleAdminRadioHighestBidder - radio:%s, bidderShow:%s",radio,(dba.bidderShow and 'true' or 'false'))
		
--~ 	self:SendAdminBlockHighestBidder()
	self:SendAdminEditOption('bidderShow')
end

function GigaItemAuction:ToggleADMINFrame()
    if self.ADMINFrame:IsVisible() then
        self:HideADMINFrame()
    else
        self:ShowADMINFrame()
    end
end

function GigaItemAuction:ShowADMINFrame()
    self.ADMINFrame:Show()
    db.config.showAdminWindow = true
end

function GigaItemAuction:HideADMINFrame()
    self.ADMINFrame:Hide()
    db.config.showAdminWindow = false
end

function GigaItemAuction:ToggleBIDLISTFrame()
    if self.BIDLISTFrame:IsVisible() then
        self:HideBIDLISTFrame()
    else
        self:ShowBIDLISTFrame()
    end
end

function GigaItemAuction:ShowBIDLISTFrame()
    self.BIDLISTFrame:Show()
end

function GigaItemAuction:HideBIDLISTFrame()
    self.BIDLISTFrame:Hide()
end





------------------------------------
--~ Send Admins

function GigaItemAuction:SendAdminBreak()
	if (not db.admin.onlyAuctioneerACP and self:CheckSecurity(PLAYERNAME, 'control')) or self:CheckSecurity(PLAYERNAME, 'auctioneer') then
		self.auctionStatusSTART = false
		self:SendAddonMSG(self:Serialize(self.amsg.key.breakauction, 1))
	else
		self:Print(L["secutity3"])
	end
end

function GigaItemAuction:SendAdminResetAuction()
	if (not db.admin.onlyAuctioneerACP and self:CheckSecurity(PLAYERNAME, 'control')) or self:CheckSecurity(PLAYERNAME, 'auctioneer') then
		self:SendAddonMSG(self:Serialize(self.amsg.key.sendresetauction, 1))
	else
		self:Print(L["secutity4"])
	end
end

function GigaItemAuction:SendAdminStartAuction()
	if (not db.admin.onlyAuctioneerACP and self:CheckSecurity(PLAYERNAME, 'control')) or self:CheckSecurity(PLAYERNAME, 'auctioneer') then
		self.auctionStatusPAUSE = false
		self:SendAddonMSG(self:Serialize(self.amsg.key.sendstartauction, 1))
	else
		self:Print(L["secutity5"])
	end
end

function GigaItemAuction:SendAdminPauseAuction()
	if (not db.admin.onlyAuctioneerACP and self:CheckSecurity(PLAYERNAME, 'control')) or self:CheckSecurity(PLAYERNAME, 'auctioneer') then
		self.auctionStatusPAUSE = true
		self:SendAddonMSG(self:Serialize(self.amsg.key.sendpauseauction, 1))
	else
		self:Print(L["secutity5"])
	end
end

function GigaItemAuction:SendAdminBlockWithdraw()
	local dbn = db.nowAuction
	if (not db.admin.onlyAuctioneerACP and self:CheckSecurity(PLAYERNAME, 'control')) or self:CheckSecurity(PLAYERNAME, 'auctioneer') then
		self:SendAddonMSG(self:Serialize(self.amsg.key.sendwithdraw, db.admin.bidWithdraw), "GUILD")
	else
		self:Print(L["secutity6"])
	end
end

function GigaItemAuction:SendAdminBlockHighestBidder()
	local dbn = db.nowAuction
	if (not db.admin.onlyAuctioneerACP and self:CheckSecurity(PLAYERNAME, 'control')) or self:CheckSecurity(PLAYERNAME, 'auctioneer') then
		self:SendAddonMSG(self:Serialize(self.amsg.key.sendanonym, db.admin.bidderShow), "GUILD")
	else
		self:Print(L["secutity7"])
	end
end

function GigaItemAuction:SendAdminEditOption(db_key, db_sub)
	local dbn = db.nowAuction
	local dba = db.admin
	local dbc = db.config
	
	if self:CheckSecurity(PLAYERNAME, 'control') or self:CheckSecurity(PLAYERNAME, 'auctioneer') then
		self:Debug("SendAdminEditOption - [%s]",(db_sub and (db_sub..'.') or '')..db_key)
		
		self:SendAddonMSG(self:Serialize(self.amsg.key.sendeditoption, db_key, db_sub and dba[db_sub][db_key] or dba[db_key], db_sub and db_sub or false), "GUILD")
	else
		self:Print(L["secutity8"])
	end
	
end

function GigaItemAuction:SendBidTimeToAll(bidTime)
	if (not db.admin.onlyAuctioneerACP and self:CheckSecurity(PLAYERNAME, 'control')) 
	or self:CheckSecurity(PLAYERNAME, 'auctioneer') then
		self:SendAddonMSG(self:Serialize(self.amsg.key.newbidtime, bidTime), "GUILD")
	end
end

function GigaItemAuction:SendAdminAuctionLonger()
	local dbn = db.nowAuction
	if self:CheckSecurity(PLAYERNAME, 'control') 
	or self:CheckSecurity(PLAYERNAME, 'auctioneer') then
		self:AdminCurrentAuctionSend(PLAYERNAME, dbn.channel or "GUILD", "restart")
	else
		self:Print(L["secutity2"])
	end
end

------------------------------
--~ Admin edition

function GigaItemAuction:AdminEditBidTimeByAdmin(value, author)	
	local dba = db.admin
	local dbc = db.config
	
	if not self:CheckSecurity(author, 'control') 
	and not self:CheckSecurity(author, 'auctioneer') then
		if dbc.debugmodus then
			self:Print("|cff5555ffDEBUG:|r |cffff5555RANK SECURITY|r - AdminEditBidTimeByAdmin. ["..author.."]")
		else
			self:Debug("|cffff5555RANK SECURITY|r - AdminEditBidTimeByAdmin. [%s]",author)
		end
		if self:CheckSecurity(PLAYERNAME, 'admin') then
			self:Print("|cffff5555HACKATTACK:|r |cffff5555RANK SECURITY|r - AdminEditBidTimeByAdmin. Hacker: ["..author.."]")
		end
		return 
	end
	
	local cauthor = self:GetColoredName(author)
	local _, _, inc_bidTime = self:Deserialize(value)
	
	dba.bidTime = tonumber(inc_bidTime)
	
	if self:CheckSecurity(PLAYERNAME, 'control') 
	or self:CheckSecurity(PLAYERNAME, 'auctioneer') then	
		self.ADMINFrame.timeSlider:SetValue(dba.bidTime)
		self:Print(L["adminedit1"]:format(cauthor, inc_bidTime))		
	elseif dbc.debugmodus then
		self:Print("|cff5555ffDEBUG:|r SET - New bidtime: "..inc_bidTime.." sec.")
	else
		self:Debug("SET - New bidtime: %ssec.",inc_bidTime)
	end
end

function GigaItemAuction:AdminAuctionLonger(author)
	if not self:CheckSecurity(author, 'control') 
	and not self.auctionStatusSTART then
		if auctionSTART then
			if dbc.debugmodus then 
				self:Print("|cff5555ffDEBUG:|r |cffff5555RANK SECURITY|r - AdminAuctionLonger. ["..author.."]")
			else
				self:Debug("|cffff5555RANK SECURITY|r - AdminAuctionLonger. [%s]",author)
			end
		end
		if self:CheckSecurity(PLAYERNAME, 'admin') then
			self:Print("|cffff5555HACKATTACK:|r |cffff5555RANK SECURITY|r - AdminAuctionLonger. Hacker: ["..author.."]")
		end
		return 
	end
	
	self:SetButtonDisable()
	
	local dbn = db.nowAuction
	local dbc = db.config
	local dba = db.admin
	
	if dbc.playsound.longauction then
		PlaySound("AuctionWindowOpen")
	end
	
--~ 	gebotbuttons kurz deaktivieren
	if dbn.last.bidderGUID == PLAYERGUID then
		self:SetBreakButtonEnable()
	else
		self:GIATimerStart("SetButtonEnable","SetButtonEnable", 1)
		self:SetBreakButtonDisable()
	end
	
--~ 	aktionsende stoppen und neu setzen
	if author == PLAYERNAME then
		self:GIATimerStart("StopAdminAuction","StopAdminAuction", dba.bidTime + 1)
	end
	self:GIATimerStart("StopAuctionButtons","SetAllButtonDisable", dba.bidTime)
	
	self:StopStatusbar()
	self:StartStatusBar()
end

function GigaItemAuction:AdminAuctionEdit(bidTime, author)
	
	self:SetButtonDisable()
	local dbn = db.nowAuction
	local dbc = db.config
	local dba = db.admin
	
	dba.bidTime = bidTime
	
--~ 	gebotbuttons kurz deaktivierenhen
	if dbn.last.bidderGUID == PLAYERGUID then
		self:SetBreakButtonEnable()
	else
		self:GIATimerStart("SetButtonEnable","SetButtonEnable", 1)
		self:SetBreakButtonDisable()
	end
	
--~ 	aktionsende stoppen und neu setzen
	if author == PLAYERNAME then
		self:GIATimerStart("StopAdminAuction","StopAdminAuction", dba.bidTime + 1)
	end
	self:GIATimerStart("StopAuctionButtons","SetAllButtonDisable", dba.bidTime)
	
	self:StopStatusbar()
	self:StartStatusBar()
end




-----------------------------------
--~ Rules database sync

function GigaItemAuction:AdminCurrentRulesSend(author, channel)
	local dbg = db.auctionSecurity.guildRank
	local dbr = db.auctionSecurity.raidRank
	local channel = channel and channel or "GUILD"
	if not self:CheckSecurity(PLAYERNAME, 'admin') then
		return
	end
	local guildSum, raidSum = self:CheckRulesSum()
--~ 	GuildRanks
	self:SendAddonMSG(self:Serialize(self.amsg.key.sendguildrules, guildSum, raidSum), channel, channel == "WHISPER" and author or nil)
	self:Debug("|cff55ff55SEND:|r Guild/Raid Rules to channel:%s", "GUILD")
end

function GigaItemAuction:GuildLeaderSyncRulesSend()
	local rank = select(4, self:GetGuildPlayer(PLAYERNAME))
	
	if IsInGuild() then
		if rank == 0 then
			self:GIATimerStart("GuildLeaderSyncRulesSend", "GuildLeaderSyncRulesSend", 2)
		elseif rank == 1 then
			self:AdminCurrentRulesSend()
		end
	end
end

function GigaItemAuction:AdminCurrentGuildRulesSync(value, author)
	if not self:CheckSecurity(author, 'auctioneer') then
		self:Debug("|cffff5555RANK SECURITY|r - AdminCurrentGuildRulesSync. [%s]","not auctioneer")
		if not self:CheckSecurity(author, 'admin') then
			self:Debug("|cffff5555RANK SECURITY|r - AdminCurrentGuildRulesSync. [%s]","not admin")
			if (select(4, self:GetGuildPlayer(author)) ~= 1) then
				self:Debug("|cffff5555RANK SECURITY|r - AdminCurrentGuildRulesSync. [%s]","not rank 1")
				if db.config.debugmodus then
					self:Print("|cff5555ffDEBUG:|r |cffff5555RANK SECURITY|r - AdminCurrentGuildRulesSync. ["..author.."]")
				else
					self:Debug("|cffff5555RANK SECURITY|r - AdminCurrentGuildRulesSync. [%s]",author)
				end
				if self:CheckSecurity(PLAYERNAME, 'admin') then
					self:Print("|cffff5555HACKATTACK:|r |cffff5555RANK SECURITY|r - AdminCurrentGuildRulesSync. Hacker: ["..author.."]")
				end
				return
			end
		end
	end

	local _, _, inc_guildSum, inc_raidSum = self:Deserialize(value)
	local my_guildSum, my_raidSum = self:CheckRulesSum()
	
	local diff_guildSum, diff_raidSum = inc_guildSum ~= my_guildSum, inc_raidSum ~= my_raidSum
	
	self:Debug("SECURITY |cff55ff55OK|r - RulesSync REFRESH: guildDiff:%s, raidDiff:%s", diff_guildSum, diff_raidSum)
	
	if diff_guildSum then
		self:RulesWithDatabaseSync(inc_guildSum, "guildRank")
	end
	if diff_raidSum then
		self:RulesWithDatabaseSync(inc_raidSum, "raidRank")
	end
	
	if diff_guildSum or diff_raidSum then
		self:SetAuctionBidStats()
	end
end
local rulesTable = {
	["guildRank"] = {"pAdmin","pAdminControl","pStart","pControl","bidMain","bidSecond","bidFun","bidU80"},
	["raidRank"] = {"pStart","pControl"}
}
function GigaItemAuction:RulesWithDatabaseSync(sumValue, dbTable)
	local dbs = db.auctionSecurity
	local rank, sum, rule = '', '', ''
	local valMatch = select(2,string.gsub(sumValue,"S",string.upper))
	local temp_old, temp_new, temp_rule
	
	-- scan complet string
	for i = 1 , string.len(sumValue), floor(string.len(sumValue)/valMatch) do
		_, _, rank, sum = string.find(sumValue,"|S(%d+)|(%w+)|",i)
		if not rank then break end
		self:Debug("|cff55ffeeRulesWithDatabaseSync:|r dbTable: %s, sum: %s, rank: %s", dbTable, sum, dbs[dbTable]["rank"..rank].name)
		-- scan rank rules
		for j = 1, string.len(sum) do
			rule = tostring(string.sub(sum,j,j))
--~ 			self:Debug(" • Rule: %s", rule)
			if not rule then break end
			temp_new = rule
			temp_old = dbs[dbTable]["rank"..rank][rulesTable[dbTable][j]] and "1" or "0"
			if temp_new ~= temp_old then
				self:Debug(" |cffff5555•|r OLD db[%s]: %s, syncRule:%s", rulesTable[dbTable][j], dbs[dbTable]["rank"..rank][rulesTable[dbTable][j]], rule)
			end
			if rule == tostring(0) then
				temp_rule = false
			elseif rule == tostring(1) then
				temp_rule = true
			else
				temp_rule = false
			end
			dbs[dbTable]["rank"..rank][rulesTable[dbTable][j]] = temp_rule
			
			if temp_new ~= temp_old then
				self:Debug(" |cff55ff55•|r NEW db[%s]: %s, tempRule:%s", rulesTable[dbTable][j], dbs[dbTable]["rank"..rank][rulesTable[dbTable][j]], temp_rule)
			end
			
		end
	end
end

function GigaItemAuction:CheckRulesSum()
	local dbg = db.auctionSecurity.guildRank
	local dbr = db.auctionSecurity.raidRank
	local raidSum, guildSum, rtemp, gtemp = '', '', {}, {}
	
	for i, v in pairs(dbr) do
		raidSum = ""
		for k = 1, #rulesTable.raidRank do
			raidSum = raidSum..(v[rulesTable.raidRank[k]] and 1 or 0)
		end
		rtemp[v.rank] = raidSum
--~ 		self:Debug("|cff55ffeeCheckRulesSum:|r RAID sum:%s, rank:%s", raidSum, i)
	end	
	
	for i, v in pairs(dbg) do
		guildSum = ""
		for k = 1, #rulesTable.guildRank do
			guildSum = guildSum..(v[rulesTable.guildRank[k]] and 1 or 0)
		end	
		gtemp[v.rank] = guildSum
--~ 		self:Debug("|cff55ffeeCheckRulesSum:|r GUILD sum:%s, rank:%s", guildSum, v.rank)
	end	
	
	raidSum = ""
	for i = 1, #rtemp do
		raidSum = raidSum.."|S"..i.."|"..rtemp[i]
	end
	raidSum = raidSum.."|."
	
	guildSum = ""
	for i = 1, #gtemp do
		guildSum = guildSum.."|S"..i.."|"..gtemp[i]
	end
	guildSum = guildSum.."|."
	
--~ 	self:Debug("|cff55ffeeCheckRulesSum:|r RAID ALL sum:%s", raidSum)
--~ 	self:Debug("|cff55ffeeCheckRulesSum:|r GUILD ALL sum:%s", guildSum)
	
	return guildSum, raidSum
end


-----------------------------------
--~ Admin database sync

function GigaItemAuction:AdminCurrentAuctionSend(author, channel, adminValue)
	local dbc = db.config
	local dba = db.admin
	self:SendAddonMSG(
		self:Serialize(
			self.amsg.key.sendbidtime, 
			dba.bidTime, 
			dba.bidUP, 
			dba.bidArt.main, 
			dba.bidArt.second, 
			dba.bidArt.fun, 
			dba.bidArt.u80, 
			dba.bidStep, 
			(adminValue and adminValue or false), 
			dba.bidWithdraw, 
			dba.bidderShow
		), 
		channel, 
		channel == "WHISPER" and author or nil
	)
	
	self:Debug("|cff55ff55SEND ADMIN DATA|r - at:%s, ch:%s",author,channel)
end

function GigaItemAuction:AdminCurrentAuctionSync(value, author)	
	local dbn = db.nowAuction
	local dbc = db.config
	local dba = db.admin
	
	if not self:CheckSecurity(author, 'control') then
		if dbc.debugmodus then
			self:Print("|cff5555ffDEBUG:|r |cffff5555RANK SECURITY|r - AdminCurrentAuctionSync. ["..author.."]")
		else
			self:Debug("|cffff5555RANK SECURITY|r - AdminCurrentAuctionSync. [%s]",author)
		end
		if self:CheckSecurity(PLAYERNAME, 'admin') then
			self:Print("|cffff5555HACKATTACK:|r |cffff5555RANK SECURITY|r - AdminCurrentAuctionSync. Hacker: ["..author.."]")
		end
		return 
	end
	
	local _, _, inc_bidTime, inc_bidUP, inc_bidArt_main, inc_bidArt_second, inc_bidArt_fun, inc_bidArt_u80, inc_bidStep, inc_AdminValue, inc_bidWithdraw, inc_bidderShow = self:Deserialize(value)			
	local edit = false
	
	self:Debug("|cffffff55SET ADMIN DATA #1|r - bt:%s, up:%s, bs:%s, av:%s",inc_bidTime, inc_bidUP, inc_bidStep, (inc_AdminValue and inc_AdminValue or "no"))
	self:Debug("|cffffff55SET ADMIN DATA #2|r - m:%s, s:%s, f:%s, u80:%s ",(inc_bidArt_main and "yes" or "no"), (inc_bidArt_second and "yes" or "no"), (inc_bidArt_fun and "yes" or "no"), (inc_bidArt_u80 and "yes" or "no"))
	self:Debug("|cffffff55SET ADMIN DATA #3|r - bw:%s, hb:%s ",(inc_bidWithdraw and "yes" or "no"), (inc_bidderShow and "no" or "yes"))
	
	if inc_bidUP ~= dba.bidUP then
		dba.bidUP = inc_bidUP
		edit = true
	end
	if inc_bidArt_main ~= dba.bidArt.main then
		dba.bidArt.main = inc_bidArt_main
		edit = true
	end
	if inc_bidArt_second ~= dba.bidArt.second then
		dba.bidArt.second = inc_bidArt_second
		edit = true
	end
	if inc_bidArt_fun ~= dba.bidArt.fun then
		dba.bidArt.fun = inc_bidArt_fun
		edit = true
	end
	if inc_bidArt_u80 ~= dba.bidArt.u80 then
		dba.bidArt.u80 = inc_bidArt_u80
		edit = true
	end
	if inc_bidStep ~= dba.bidStep then
		dba.oldBidStep = dba.bidStep
		dba.bidStep = inc_bidStep
		edit = true
	end
	if inc_bidWithdraw ~= dba.bidWithdraw then
		dba.bidWithdraw = inc_bidWithdraw
		edit = true
	end
	if inc_bidderShow ~= dba.bidderShow then
		dba.bidderShow = inc_bidderShow
		edit = true
	end
	if edit or inc_AdminValue == "restart" then
		self:SyncButtonsWithDatabase()
	end
	if inc_bidTime ~= dba.bidTime or inc_AdminValue == "restart" then
		self:AdminAuctionEdit(inc_bidTime, author)
	end
end

-------------------------------------
--~ Security Priority Check

function GigaItemAuction:CheckSecurity(p_name, p_mode)
	local rankDBName = "rank"..select(4, self:GetGuildPlayer(p_name))
	local _, isLead, isAssist, isML = self:GetRaidPlayer(p_name)
	local dbg = db.auctionSecurity.guildRank
	local dbr = db.auctionSecurity.raidRank
	local dbn = db.nowAuction
	local security = false
	
	if rankDBName == "rank0" then
		self:Debug("|cffff0000ERROR:|r Fail CheckSecurity. Player:%s, Mode:%s",p_name,p_mode)
		return false
	elseif p_mode == 'admin' and dbg[rankDBName].pAdmin then
		security = true
	elseif p_mode == 'admincontrol' and dbg[rankDBName].pAdminControl then
		security = true
	elseif p_mode == 'control' then
		if dbg[rankDBName].pControl then
			security = true
		end
		if not security then
			if isML and dbr.rank3.pControl then
				security = true
			end			
			if not security then
				if isLead and dbr.rank1.pControl then
					security = true
				elseif isAssist and dbr.rank2.pControl then
					security = true
				end
			end
		end
	elseif p_mode == 'start' then
		if dbg[rankDBName].pStart then
			security = true
		end
		if not security then
			if isML and dbr.rank3.pStart then
				security = true
			end			
			if not security then
				if isLead and dbr.rank1.pStart then
					security = true
				elseif isAssist and dbr.rank2.pStart then
					security = true
				end
			end
		end
	elseif p_mode == 'auctioneer' then
		if p_name == dbn.auctionAuthor then
			security = true
		end
	end
	
	self:Debug("CheckSecurity - pl:%s, md:%s, rn:%s, sc:%s", p_name, p_mode, rankDBName, security)
	
	return security
end

function GigaItemAuction:CheckSecurityBid(p_name, p_mode)
	local rankDBName = "rank"..select(4, self:GetGuildPlayer(p_name))
	local dbg = db.auctionSecurity.guildRank
	local dbn = db.nowAuction
	local priority = false
	
	if rankDBName == "rank0" then
		self:Debug("|cffff0000ERROR:|r Fail CheckSecurityBid. Player:%s, Mode:%s",p_name,p_mode)
		return false
	elseif p_mode == self.BidArt.kurz[1] then
		priority = dbg[rankDBName].bidMain
	elseif p_mode == self.BidArt.kurz[2] then
		priority = dbg[rankDBName].bidSecond
	elseif p_mode == self.BidArt.kurz[3] then
		priority = dbg[rankDBName].bidFun
	elseif p_mode == 'u80' then
		priority = dbg[rankDBName].bidU80
	end
	
	self:Debug("CheckSecurityBid: pl:%s, rn:%s, pm:%s, prio:%s", p_name, rankDBName, p_mode, priority)
	
	return priority
end

----------------------------------
--~ Admin control auction

function GigaItemAuction:StartAdmin()
	if not self:CheckSecurity(PLAYERNAME, 'admin') then return end
	local dbn = db.nowAuction
	local dba = db.admin
	local a = self.ADMINFrame
	
	if dba.onlyAuctioneerACP and not self:CheckSecurity(PLAYERNAME, 'auctioneer') then
		return
	end
		
	self:ShowADMINFrame()
	
	if self:CheckSecurity(PLAYERNAME, 'auctioneer') then
		self:ShowBIDLISTFrame()
		a.adminButton:Enable()
	else
		self:HideBIDLISTFrame()
		a.adminButton:Disable()
	end
	a.breakButton:Enable()
	a.pauseButton:Enable()
	a.resetButton:Enable()
	a.startButton:Disable()
	a.aLongButton:Enable()
end

function GigaItemAuction:StopAdmin()
	local dbn = db.nowAuction
	local dbc = db.config
	local dba = db.admin
	local a = self.ADMINFrame
	
	if dbn.auctionAuthor ~= PLAYERNAME then
		self:HideADMINFrame()
		a.adminButton:Enable()
	end
	a.breakButton:Disable()
	a.pauseButton:Disable()
	a.resetButton:Disable()
	a.startButton:Disable()
	a.aLongButton:Disable()
end






